package com.cringe_studios.cringe_authenticator_library;

import java.security.InvalidKeyException;

import com.cringe_studios.cringe_authenticator_library.impl.Base32;

public abstract class OTP {
	//TOTP
	//HOTP
	
	protected byte[] secret;
	protected String base32Secret;
	protected OTPType type;
	protected OTPAlgorithm algorithm;
	protected int digits;
	protected long counter;
	protected long period;
	protected boolean checksum;
	static protected long timeOffset;
	
	protected OTP(OTPType nType, String nSecret, OTPAlgorithm nAlgorithm, int nDigits, long nCounter, long nPeriodInSeconds, boolean nChecksum) {
		this.secret = Base32.decode(nSecret);
		base32Secret = nSecret;
		type = nType;
		algorithm = (nAlgorithm == null) ? OTPAlgorithm.SHA1 : nAlgorithm;
		digits = (nDigits <= 0) ? 6 : nDigits;
		counter = nCounter;
		period = (nPeriodInSeconds <= 0) ? 30 : nPeriodInSeconds;
		checksum = nChecksum;
	}
	
	//secret: required - base32 encoded
	//algorithm: optional (default SHA1)
	//digits: optional (default 6)
	//OTPType: required
	//period: optional (default 30)
	//counter: required if hotp
	
	/**
	 * 
	 * @param type REQUIRED: the type of the OTP (HTOP or TOTP)
	 * @param secret REQUIRED: the secret, base32 encoded
	 * @param algorithm OPTIONAL (default SHA1) the hash algorithm to use (SHA1 SHA224 SHA256 SHA384 SHA512)
	 * @param digits OPTIONAL (Default 6) how many digits to calculate
	 * @param counter REQUIRED if HOTP, otherwise meaningless. The initial counter for HOTP (eg 21, if user reloaded counter 21 times)
	 * @param periodInSeconds OPTIONAL (default 30) for TOTP, the refresh rate of the TOTP
	 * @param checksum OPTIONAL: appends a checksum digit to the end of the string
	 * @return returns a valid OTP object, which can be used to create a new OTP pin
	 */ 
	public static OTP createNewOTP(OTPType type, String secret, OTPAlgorithm algorithm, int digits, long counter, long periodInSeconds, boolean checksum) throws OTPException{
		if(type == null) throw new OTPException("No OTP Type given!");
		return type.instance(secret, algorithm, digits, counter, periodInSeconds, checksum);
	}
	
	public abstract long getNextDueTime();
	
	/**
	 * 
	 * @return s the current pin, as calculated by the counter or the current time, unless it
	 * @throws an OTPException
	 */
	public abstract String getPin() throws OTPException;
	
	public abstract void incrementCounter();
	
	public String getSecret() {
		return this.base32Secret;
	}
	
	public OTPType getOTPType() {
		return this.type;
	}
	
	public OTPAlgorithm getAlgorithm() {
		return algorithm;
	}
	
	public int getDigits() {
		return this.digits;
	}
	
	public long getCounter() {
		return this.counter;
	}
	
	public long getPeriod() {
		return this.period;
	}
	
	public boolean hasChecksum() {
		return checksum;
	}
	
	// Seconds towards positive infinity means a positive correction into the future (seconds = 2 -> 5 becomes 7)
	// Seconds towards negative infinity means a negative correction into the past   (seconds = -2 -> 5 becomes 3)
	public static void setTimeCorrection(long seconds) {
		timeOffset = seconds;
	}
	
	public static long getTimeCorrection() {
		return timeOffset;
	}
}